"""
Copyright (C) 2005 Magnus Bjernstad <bjernstad@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import gtk, math

FREQS = ["31.25 ","62.5 ","125 ","250 ","500 ","1 k","2 k","4 k","8 k","16 k"]
N_BANDS = len(FREQS)

def db_to_gain(db):
    return 10**(db/20.0)

def gain_to_db(gain):
    return 20*math.log10(gain)

class Equalizer(gtk.VBox):
    def __init__(self, player):
        gtk.VBox.__init__(self)
        self.player = player

        hbox_bands = gtk.HBox()
        self.enable_check_button = gtk.CheckButton("Enabled")
        self.enable_check_button.connect("toggled", self.on_enabled_toggled)

        self.eq_bands = []
        for i in range(N_BANDS):
            band = Band(i)
            band.scale.connect("adjust-bounds", self.on_adjust_bounds, i)
            hbox_bands.pack_start(band, True, True)
            self.eq_bands.append(band)
            self.player.config_observer.add("effect.equalizer.gain%d" % (i), self.new_eq_setting_cb)
            self.player.client.configval_get("effect.equalizer.gain%d" % (i), GetGainHandler(self, i))

        self.player.config_observer.add("effect.equalizer.enabled", self.enabled_changed_cb)

        button = gtk.Button("Reset")
        button.connect("clicked", self.on_reset_clicked)
        # Pack widgets
        self.pack_start(hbox_bands)
        hbox_bottom = gtk.HBox()
        hbox_bottom.pack_start(self.enable_check_button, False, False)
        hbox_bottom.pack_end(button, False, False)
        #self.pack_start(self.enable_check_button, False, False)
        self.pack_start(hbox_bottom, False, False)

        self.show_all()

        self.player.client.configval_get("effect.equalizer.enabled", self.query_enabled_cb)

    def destructor(self):
        for i in range(N_BANDS):
            self.player.config_observer.remove("effect.equalizer.gain%d" % (i), self.new_eq_setting_cb)
        self.player.config_observer.remove("effect.equalizer.enabled", self.enabled_changed_cb)

    ######################
    # Client lib callbacks
    ######################

    def query_enabled_cb(self, res):
        enabled = bool(int(res.value()))
        self.enable_check_button.set_active(enabled)

    ####################
    # Settings callbacks
    ####################

    def new_eq_setting_cb(self, key, value):
        band_number = int(key[-1])
        value = gain_to_db(float(value))
        band = self.eq_bands[band_number]
        if value != band.get_value():
           band.set_value(value)

    def enabled_changed_cb(self, key, value):
        value = bool(int(value))
        self.enable_check_button.set_active(value)

    #################
    # Event handlers
    #################

    def on_adjust_bounds(self, widget, value, band_number):
        key = "effect.equalizer.gain%d" % (band_number)
        gain = "%.1f" % db_to_gain(value)
        self.player.client.configval_set(key, gain)

    def on_enabled_toggled(self, togglebutton):
        new_value = togglebutton.get_active()
        new_value = str(int(new_value))
        self.player.client.configval_set("effect.equalizer.enabled", new_value)

    def on_reset_clicked(self, widget):
        for i in range(N_BANDS):
            key = "effect.equalizer.gain%d" % (i)
            self.player.client.configval_set(key, "1.0")

class Band(gtk.VBox):
    def __init__(self, band):
        gtk.VBox.__init__(self)

        label = gtk.Label(FREQS[band] + "Hz")
        self.scale = gtk.VScale()
        self.scale.set_range(-8.0, 8.0)
        self.scale.set_range(-8.0, 8.0)
        self.scale.set_increments(0.05, 0.05)
        self.scale.set_inverted(True)
        self.scale.connect("format-value", self.on_format_value)

        self.pack_start(self.scale, True, True)
        self.pack_start(label, False, True)

    def set_value(self, value):
        self.scale.set_value(value)
    
    def get_value(self):
        return self.scale.get_value()

    def on_format_value(self, widget, value):
        return "%+.1f dB" % (value)

class GetGainHandler:
    def __init__(self, caller, band):
        self.caller = caller
        self.band = band

    def __call__(self, res):
        value = float(res.value())
        value = gain_to_db(value)
        self.caller.eq_bands[self.band].set_value(value)

