"""
Copyright (C) 2005 Magnus Bjernstad <bjernstad@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import gtk, gtk.glade, os, functions, song_tree_view, xml.sax.saxutils, urllib, gobject

ENTITY_TYPE_FILE = 0
ENTITY_TYPE_DIR = 1
ENTITY_TYPE_PARENT = 2

class FileBrowser(gtk.VBox):
    def __init__(self, player):
        gtk.VBox.__init__(self)
        self.player = player
        self.song_view = song_tree_view.SongView(player, "file_browser")

        glade_file = functions.glade_file("file_browser")
        self.gxml_menu = gtk.glade.XML(glade_file, "popup_menu")
        self.gxml_menu.signal_autoconnect(self)

        # Event handlers
        self.song_view.view.connect("row-activated", self.on_row_activated)
        self.song_view.view.connect("button-press-event", self.on_button_press)
        self.song_view.view.connect("button-release-event", self.on_button_release)
        self.song_view.view.connect("popup-menu", self.on_popup_menu)

        self.model = gtk.ListStore(object, int)
        self.song_view.view.set_model(self.model)

        self.cwd_label = gtk.Label()
        self.cwd_label.set_alignment(0, 4)
        self.scrolled = gtk.ScrolledWindow()
        
        self.cell_pixbuf = gtk.CellRendererPixbuf()
        self.cell_text = gtk.CellRendererText()

        # Pack widgets
        self.pack_start(self.cwd_label, False, False, 3)
        self.pack_start(self.song_view)

        player.settings.add_cb("file_browser", "extensions", self.extensions_cb)

        self.browse_state = BrowseState(player)
        self.browse_state.connect("changed", self.update_path_label)
        self.browse_state.connect("changed", self.on_browse_state_changed)

        # Initial update
        player.settings.emit("file_browser", "extensions")

    def update_path_label(self, source, cwd, focus_row):
        self.cwd_label.set_markup(cwd)

    def show_menu(self, button=0, time=0):
        w = self.gxml_menu.get_widget
        selected = self.song_view.view.get_selection().count_selected_rows() > 0
        w("menu_play").set_sensitive(selected)
        w("menu_add").set_sensitive(selected)
        w("popup_menu").popup(None, None, None, button, time)

    def on_browse_state_changed(self, source, cwd, focus_row):
        entries = os.listdir(cwd)
        entries = filter(lambda x: x[0] != ".", entries)

        self.model.clear()

        if cwd != "/":
            self.model.append(({"dir" : os.pardir}, ENTITY_TYPE_PARENT))

        # First split the dirs from the files to make sure all dirs are added
        # before the files and that they are sorted.
        dirs = []
        files = []

        for entry in entries:
            if os.path.isdir(os.path.join(cwd, entry)):
                dirs.append(entry)
            else:
                root, ext = os.path.splitext(entry)
                if ext.lower() in self.extensions or not self.extensions:
                    files.append(entry)

        # Add the sorted dirst
        dirs.sort(key=str.lower)
        for dir in dirs:
            self.model.append(({"dir" : dir}, ENTITY_TYPE_DIR))

        # Add the sorted files
        files.sort(key=str.lower)
        for file in files:
            self.model.append(({"file" : file}, ENTITY_TYPE_FILE))

        if focus_row:
            self.song_view.view.set_cursor(focus_row)

    def activate_row(self, path):
        iter = self.model.get_iter(path)
        dict = self.model.get_value(iter, 0)
        browse_type = self.model.get_value(iter, 1)

        if browse_type == ENTITY_TYPE_DIR:
            self.browse_state.push(dict["dir"], path)

        elif browse_type == ENTITY_TYPE_PARENT:
            self.browse_state.pop()

        elif browse_type == ENTITY_TYPE_FILE:
            self.gxml_menu.get_widget("menu_add").activate()

    def add_selected(self):
        self.__add_selected()

    def play_selected(self):
        self.player.client.playlist_clear()
        self.__add_selected(self.start_playback)

    def start_playback(self):
        self.player.client.playlist_set_next(0)
        self.player.client.playback_tickle()
        self.player.client.playback_start()

    def __add_selected(self, callback=None):
        self.callback = callback
        model, paths = self.song_view.view.get_selection().get_selected_rows()
        cwd = self.cwd_label.get_text()

        # TODO: Find a less ugly way to detect when everything has been added.
        for path in paths:
            entity_type = self.model[path][1]
            selected = self.model[path][0]

            if entity_type == ENTITY_TYPE_FILE:
                url = "file://" + os.path.join(cwd, selected["file"])
                self.player.client.playlist_add(url)
            elif entity_type == ENTITY_TYPE_DIR:
                path = os.path.join(cwd, selected["dir"])
                for dirpath, dirnames, filenames in os.walk(path):
                    filenames.sort()
                    for filename in filenames:
                        to_add = "file://" + os.path.join(dirpath, filename)
                        self.player.client.playlist_add(to_add)
        functions.queue_function(self.player, self.callback)

    def destructor(self):
        self.song_view.destructor()
        self.browse_state.destructor()
        self.player.settings.remove_cb("file_browser", "extensions", self.extensions_cb)

    ####################
    # Settings callbacks
    ####################

    def extensions_cb(self, section, option, value):
        self.extensions = value.split(",")
        self.extensions = filter(lambda x: x != "", self.extensions)
        self.extensions = ["." + ext.lower() for ext in self.extensions]
        self.browse_state.notify()

    ################
    # Event handlers
    ################
    def on_row_activated(self, treeview, path, view_column):
        self.activate_row(path)

    def on_button_press(self, widget, event):
        if event.button == 3:
            return self.song_view.view.get_selection().count_selected_rows() > 1

    def on_button_release(self, widget, event):
        if event.button == 3:
            self.show_menu(event.button, event.time)
        elif event.button == 2:
            try:
                if self.model[0][1] == ENTITY_TYPE_PARENT:
                    self.activate_row(0)

            except IndexError:
                print "Warning: Model is empty."
        return True

    def on_popup_menu(self, widget):
        self.show_menu()
        
    ################
    # Menu callbacks
    ################
    def on_menu_play(self, menuitem):
        self.play_selected()

    def on_menu_add(self, menuitem):
        self.add_selected()

    def on_menu_refresh(self, menuitem):
        self.browse_state.notify()


class BrowseState(gobject.GObject):
    """
    Keeps track of the current position in the file hierarchy.
    The ones interested can connect to the 'changed' signal, which is emitted
    when a new place in the file hierarchy has been selected.
    """
    __gsignals__ = {
        "changed" : (gobject.SIGNAL_RUN_LAST,
                gobject.TYPE_NONE,
                (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT))
    }

    def __init__(self, player):
        gobject.GObject.__init__(self)
        player.settings.add_cb("file_browser", "remember_cwd", self.remember_cwd_cb)
        self.player = player
        self.cwd = player.settings.get("file_browser", "cwd")
        self.cwd = os.path.expanduser(self.cwd)
        self.paths = []
        self.notify()

    def push(self, value, path):
        new_cwd = os.path.join(self.cwd, value)
        if os.access(new_cwd, os.R_OK):
            self.cwd = new_cwd
            self.save_cwd()
            self.paths.append(path)
            self.notify()

    def pop(self):
        self.cwd = os.path.split(self.cwd)[0]
        self.save_cwd()
        focus_row = None
        try:
            focus_row = self.paths.pop()
        except IndexError:
            pass
        self.notify(focus_row)

    def notify(self, focus_row=None):
        self.emit("changed", self.cwd, focus_row)

    def remember_cwd_cb(self, section, option, value):
        self.save_cwd()

    def save_cwd(self):
        if self.player.settings.get("file_browser", "remember_cwd") == "1":
            self.player.settings.set("file_browser", "cwd", self.cwd)

    def destructor(self):
        self.player.settings.remove_cb("file_browser", "remember_cwd", self.remember_cwd_cb)


if gtk.check_version(2, 8, 0):
    gobject.type_register(BrowseState)
