"""
Copyright (C) 2005 Magnus Bjernstad <bjernstad@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import gtk, gtk.glade, song_tree_view, sys, functions, gobject

class Find(gtk.VBox):
    FIND_FIELDS = ["album", "title", "url", "bitrate", "artist", "tracknr", "date", "mime", "genre", "any"]

    def __init__(self, player):
        gtk.VBox.__init__(self)
        self.player = player
        self.song_view = song_tree_view.SongView(player, "find")

        glade_file = functions.glade_file("find")
        self.gxml_menu = gtk.glade.XML(glade_file, "find_popup")
        self.gxml_menu.signal_autoconnect(self)
        
        self.song_view.view.connect("row-activated", self.on_row_activated)
        self.song_view.view.connect("button-press-event", self.on_button_press)
        self.song_view.view.connect("button-release-event", self.on_button_release)

        self.model = gtk.ListStore(object)
        self.song_view.view.set_model(self.model)
        self.keypress_queue = 0

        # Init widgets
        self.hbox = gtk.HBox()
        self.entry = gtk.Entry()
        self.combobox = gtk.combo_box_new_text()

        for field in self.FIND_FIELDS:
            self.combobox.append_text(field.title())

        # Restore last search field
        active_field = self.player.settings.get("find", "field")
        try:
            active_index = self.FIND_FIELDS.index(active_field)
        except ValueError:
            active_index = 0
        self.combobox.set_active(active_index)

        # Connect signals
        self.entry.connect("changed", self.on_changed)
        self.entry.connect("activate", self.on_activate)
        self.entry.connect("key-press-event", self.on_key_pressed)
        self.combobox.connect("changed", self.on_changed)

        # Pack widgets
        self.pack_start(self.hbox, False, False)
        self.pack_start(self.song_view)
        self.hbox.pack_start(self.entry)
        self.hbox.pack_start(self.combobox, False, False)

    def show_menu(self, button=0, time=0):
        w = self.gxml_menu.get_widget
        selected = self.song_view.view.get_selection().count_selected_rows() > 0
        w("menu_play").set_sensitive(selected)
        w("menu_add").set_sensitive(selected)
        w("find_popup").popup(None, None, None, button, time)

    def destructor(self):
        self.song_view.destructor()

    def add_selected(self):
        self.__add_selected()

    def play_selected(self):
        self.player.client.playlist_clear()
        self.__add_selected(self.start_playback)

    def __add_selected(self, callback=None):
        """
        Add the selected ids to the playlist and call 'callback' when done.
        """
        model, paths = self.song_view.view.get_selection().get_selected_rows()

        for path in paths:
            id = int(self.model[path][0]["id"])
            self.player.client.playlist_add_id(id)

        functions.queue_function(self.player, callback)

    def start_playback(self):
        self.player.client.playlist_set_next(0)
        self.player.client.playback_tickle()
        self.player.client.playback_start()

    ################
    # Event handlers
    ################

    def on_row_activated(self, treeview, path, view_column):
        self.gxml_menu.get_widget("menu_add").activate()

    def on_button_press(self, widget, event):
        if event.button == 3:
            return self.song_view.view.get_selection().count_selected_rows() > 1

    def on_button_release(self, widget, event):
        if event.button == 3:
            self.show_menu(event.button, event.time)

    def on_changed(self, widget):
        new_field = self.combobox.get_active_text().lower()
        self.player.settings.set("find", "field", new_field)

        if self.player.settings.get("find", "incremental") == "1" or widget == self.combobox:
            self.keypress_queue += 1
            gobject.timeout_add(200, self.timeout_search)

    def timeout_search(self):
        self.keypress_queue -= 1
        if self.keypress_queue <= 0:
            self.entry.activate()

    def on_activate(self, widget):
        what = self.entry.get_text()

        # Require at least two characters
        if len(what) < 2:
            return

        what = what.replace("'", "''")

        # Find query type
        field = self.combobox.get_active_text().lower()

        if field == 'any':
            values = what.split(" ")
            nv = len(values)

            q  = "SELECT m1.id"
            q += " FROM "
            q += ", ".join(["media m%d" % (i,) for i in range(1,nv+1)])

            if nv > 0:
                q += " WHERE "
            q += " AND ".join(["m%d.id=m1.id" % (i,) for i in range(2,nv+1)])

            if nv > 1:
                q += " AND "

            q += " AND ".join(["m%d.value LIKE '%%%s%%'" %
                    (i, v) for (i,v) in zip(range(1,nv+1), values)])
            q += " GROUP BY m1.id"

        else: # standard key field search
            q = "SELECT id FROM media WHERE key='%s' AND value LIKE '%%%s%%'" % ((field, what))
        self.player.client.medialib_select(q, self.query_cb)


    def on_key_pressed(self, widget, event):
        # Add all matches to the playlist
        if event.keyval == gtk.gdk.keyval_from_name("Return") and \
                event.state & gtk.gdk.CONTROL_MASK:
                    self.song_view.view.get_selection().select_all()
                    self.__add_selected()

    ######################
    # Client lib callbacks
    ######################

    def query_cb(self, res):
        dicts = res.value()
        if dicts == None:
            return
        get_info = self.player.client.medialib_get_info

        # Fill in the results
        self.song_view.view.set_model(None)

        self.model.clear()

        for dict in dicts:
            try:
                get_info(int(dict["id"]), self.append_row_cb)
            except KeyError, e:
                print >>sys.stderr, "Find: Entry doesn't contain 'id' metadata"
        functions.queue_function(self.player, self.song_view.view.set_model, self.model)

    def append_row_cb(self, res):
        data = res.value()
        self.model.append((data,))

    ################
    # Menu callbacks
    ################
    def on_add_activate(self, widget):
        self.add_selected()

    def on_play_activate(self, widget):
        self.play_selected()

    def on_info(self, widget):
        model, rows = self.song_view.view.get_selection().get_selected_rows()
        if len(rows) > 0:
            row = rows[0]
            info_map = self.model[row][0]
            song_tree_view.TagInfo(info_map)

