"""
Copyright (C) 2005 Magnus Bjernstad <bjernstad@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import os
import xml.sax.saxutils
import functions

ROOT = 1
TEXT = 2
FIELD = 3
OPEN_BRACKET = 4
CLOSE_BRACKET = 5
AND = 6
OR = 7
GROUP = 8

map = { 1: "ROOT",
        2: "TEXT",
        3: "FIELD",
        4: "OPEN_BRACKET",
        5: "CLOSE_BRACKET",
        6: "AND",
        7: "OR",
        8: "GROUP" }

escape = xml.sax.saxutils.escape

class Lexer:
    def __init__(self, input):
        self.input = input
        self.index = 0

    def next(self):
        text = ""
        while 1:
            if self.index >= len(self.input):
                if text != "":
                    return((TEXT, text))
                return None
            char = self.input[self.index]
            if char in ("|", "&", "[", "]", "%"):
                if text != "":
                    return((TEXT, text))
                    text = ""
            if char == "#" and self.index+1 < len(self.input):
                c = self.input[self.index+1]
                self.index += 2
                return ((TEXT, c))

            if char == "|":
                self.index += 1
                return((OR, ""))

            if char == "&":
                self.index += 1
                return((AND, ""))

            if char == "&":
                self.index += 1
                return((AND, ""))

            if char == "[":
                self.index += 1
                return((OPEN_BRACKET, ""))

            if char == "]":
                self.index += 1
                return((CLOSE_BRACKET, ""))

            if char == "%":
                self.index += 1
                end_index = self.input.find("%", self.index)
                if end_index >= 0:
                    field = self.input[self.index:end_index]
                    self.index += len(field) + 1
                    return((FIELD, field))

            text += char
            self.index += 1

class Format:
    def __init__(self, format):
        self.set_format(format)

    def add(self, current, token):
        try:
            type, value = token
        except TypeError:
            return

        if type == TEXT:
            node = Node(current, type, value)
            current.children.append(node)

        elif type == FIELD:
            if value.find(":") >= 0:
                source, key = value.split(":")
                value = (source, key)
            node = Node(current, type, value)
            current.children.append(node)

        elif type == AND:
            node = Node(current, type, value)
            node.children.append(current.children[-1])
            current.children.remove(current.children[-1])
            current.children.append(node)
            self.add(node, self.lexer.next())

        elif type == OR:
            node = Node(current, type, value)
            node.children.append(current.children[-1])
            current.children.remove(current.children[-1])
            current.children.append(node)
            self.add(node, self.lexer.next())

        elif type == OPEN_BRACKET:
            node = Node(current, GROUP, value)
            current.children.append(node)
            while 1:
                ctoken = self.lexer.next()
                try:
                    ctype, cvalue = ctoken
                except TypeError:
                    break
                if ctype == CLOSE_BRACKET:
                    break
                self.add(node, ctoken)

    def set_format(self, format):
        format = format.replace(os.linesep, "")
        format = format.replace("\\n", os.linesep)
        self.lexer = Lexer(format)
        self.parse()

    def parse(self):
        self.root = Node(None, ROOT, None)
        current = self.root
        
        while 1:
            token = self.lexer.next()
            if token == None:
                break
            self.add(current, token)
        return

    def format(self, info):
        self.info = info
        return self.visit(self.root)

    def visit(self, node):

        if node.type == FIELD:
            if self.info.get(node.value) != None:
                if node.value == ("server", "duration"):
                    return True, functions.format_time(self.info[node.value])
                elif node.value == ("server",  "bitrate"):
                    return True, "%d kbps" % (self.info[node.value]/1000,)
                else:
                    return True, escape(str(self.info[node.value]))
            else:
                return False, str(node.value)

        elif node.type == TEXT:
            return None, node.value

        elif node.type == AND:
            string = ""
            for child in node.children:
                res = self.visit(child)
                string += res[1]
                if res[0] == False:
                    return False, ""
            return True, string

        elif node.type == OR:
            for child in node.children:
                res = self.visit(child)
                if res[0] == True:
                    return True, res[1]
            return False, ""

        elif node.type == GROUP:
            string = ""
            found = False
            for child in node.children:
                res = self.visit(child)
                string += res[1]
                if res[0] == True:
                    found = True
            if found == True:
                return True, string
            else:
                return False, ""

        elif node.type == ROOT:
            string = ""
            for child in node.children:
                res = self.visit(child)
                string += res[1]
            return string

class Node:
    def __init__(self, parent, type, value):
        self.children = []
        self.parent = parent
        self.type = type
        self.value = value

    def add_child(self, child):
        self.children.append(child)

    def get_parent(self):
        return self.parent


def show(node, level=0):
    print " "*3*level, map[node.type], node.value
    for child in node.children:
        show(child, level+1)

