"""
Copyright (C) 2005 Magnus Bjernstad <bjernstad@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import gtk, xmmsclient, os, functions, format

def add_gui_widget(player, name, box):
    w = None

    if name == "Previous":
        w = PrevButton(player)
    elif name == "Play":
        w = PlayButton(player)
    elif name == "Stop":
        w = StopButton(player)
    elif name == "Next":
        w = NextButton(player)
    elif name == "Repeat":
        w = RepeatButton(player)
    elif name == "Time":
        w = TimeLabel(player)
    elif name == "Preferences":
        w = PreferencesButton(player)

    if w != None:
        box.pack_start(w, False, False)
        return

    if name == "Seek bar":
        w = SeekBar(player)
    elif name == "Volume":
        w = VolumeScale(player)

    if w != None:
        box.pack_start(w, True, True)


class ShellacButton(gtk.Button):
    """
    Base class setting a few commong properties of the widgets.
    """
    def __init__(self, player):
        gtk.Button.__init__(self)
        self.player = player
        self.image = gtk.Image()
        self.set_relief(gtk.RELIEF_NONE)
        self.add(self.image)
        self.connect("clicked", self.on_clicked)

    def set_resource(self, value):
        abs_path = functions.get_abs_path(os.path.join("images", value))
        self.image.set_from_file(abs_path)


class ShellacToggleButton(gtk.ToggleButton):
    def __init__(self, player):
        gtk.ToggleButton.__init__(self)
        self.player = player
        self.image = gtk.Image()
        self.set_relief(gtk.RELIEF_NONE)
        self.add(self.image)
        self.connect("toggled", self.on_toggled)

    def set_resource(self, value):
        abs_path = functions.get_abs_path(os.path.join("images", value))
        self.image.set_from_file(abs_path)


class PreferencesButton(ShellacButton):
    def __init__(self, player):
        ShellacButton.__init__(self, player)
        self.image.set_from_stock(gtk.STOCK_PREFERENCES, gtk.ICON_SIZE_SMALL_TOOLBAR)

    def on_clicked(self, widget):
        import settingseditor
        settingseditor.SettingsEditor(self.player)


class PrevButton(ShellacButton):
    def __init__(self, player):
        ShellacButton.__init__(self, player)
        self.set_resource("prev.png")

    def on_clicked(self, widget):
        self.player.client.playlist_set_next_rel(-1)
        self.player.client.playback_tickle()


class PlayButton(ShellacButton):
    def __init__(self, player):
        ShellacButton.__init__(self, player)
        self.set_resource("play.png")

    def on_clicked(self, widget):
        state = self.player.get_property("status")
        if state in (xmmsclient.PLAYBACK_STATUS_STOP, xmmsclient.PLAYBACK_STATUS_PAUSE):
            self.player.client.playback_start()
        else:
            self.player.client.playback_pause()


class StopButton(ShellacButton):
    def __init__(self, player):
        ShellacButton.__init__(self, player)
        self.set_resource("stop.png")

    def on_clicked(self, widget):
        self.player.client.playback_stop()


class NextButton(ShellacButton):
    def __init__(self, player):
        ShellacButton.__init__(self, player)
        self.set_resource("next.png")

    def on_clicked(self, widget):
        self.player.client.playlist_set_next_rel(1)
        self.player.client.playback_tickle()


class RepeatButton(ShellacToggleButton):
    def __init__(self, player):
        ShellacToggleButton.__init__(self, player)
        self.set_resource("repeat.png")
        self.player.config_observer.add("playlist.repeat_all", self.on_conf_changed)
        self.player.config_observer.emit("playlist.repeat_all")

    def on_toggled(self, widget):
        new_value = widget.get_active()
        if new_value == True:
            new_value = "1"
        else:
            new_value = "0"
        self.player.client.configval_set("playlist.repeat_all", new_value)

    def on_conf_changed(self, key, value):
        value = bool(int(value))
        self.set_active(value)

    def destructor(self):
        self.player.config_observer.remove("playlist.repeat_all", self.on_conf_changed)


class VolumeScale(gtk.HScale):
    def __init__(self, player):
        gtk.HScale.__init__(self)
        self.player = player
        self.msg_sent = False

        self.set_range(0, 100)
        self.set_increments(2, 1)
        self.set_value_pos(gtk.POS_LEFT)

        self.connect("format-value", self.on_format_value)
        self.connect("value-changed", self.on_value_changed)

        self.connect("notify::volume", self.volume_changed_cb)

    def volume_changed_cb(self, obj, property):
        vol = self.player.get_property("volume")
        if not self.msg_sent:
            new_value = int((vol["left"] + vol["right"])/2)
            old_value = int(self.get_value())
            if new_value != old_value:
                self.set_value(new_value)
        else:
            self.msg_sent = False

    def on_format_value(self, widget, value):
        return "Volume:"

    def on_value_changed(self, widget):
        value = int(self.get_value())
        self.msg_sent = True
        self.player.client.playback_volume_set("left", value)
        self.player.client.playback_volume_set("right", value)

    def destructor(self):
        pass


class TimeLabel(gtk.EventBox):
    def __init__(self, player):
        gtk.EventBox.__init__(self)
        self.player = player

        self.player.connect("notify::playtime", self.update)
        self.player.connect("notify::duration", self.update)
        self.connect("button-press-event", self.on_button_pressed)

        self.label = gtk.Label()
        self.add(self.label)

        self.time_format = self.player.settings.get("gui", "time_format")
        self.player.settings.add_cb("gui", "time_format", self.time_format_cb)
        self.f = format.Format(self.time_format)

        self.dict = {
                "total"     : 0,
                "elapsed"   : 0,
                "remaining" : 0}

    def update(self, obj, property, force=False):
        # First decide as quickly as possible if we need to update
        elapsed = self.player.get_property("playtime")
        e = functions.format_time(elapsed, "%02d:%02d")

        if self.dict["elapsed"] == e and force==False:
            # No update needed
            return
        
        # Update needed
        total = self.player.get_property("duration")
        remaining = elapsed - total

        self.dict["elapsed"] = e
        self.dict["total"] = functions.format_time(total, "%02d:%02d")
        self.dict["remaining"] = functions.format_time(remaining, "%02d:%02d")

        self.label.set_markup(self.f.format(self.dict))

    def time_format_cb(self, section, option, value):
        self.f.set_format(value)
        self.update(None, None, True)

    def on_button_pressed(self, widget, event):
        f = self.time_format.find
        r = self.time_format.replace

        if f("%elapsed%") >= 0 and f("%remaining%") < 0:
            self.time_format = r("%elapsed%", "%remaining%")
        elif f("%remaining%") >= 0 and f("%elapsed%") < 0:
            self.time_format = r("%remaining%", "%elapsed%")

        self.player.settings.set("gui", "time_format", self.time_format)
        self.update(None, None)

    def destructor(self):
        self.player.settings.remove_cb("gui", "time_format", self.time_format_cb)


class SeekBar(gtk.HScale):
    def __init__(self, player):
        gtk.HScale.__init__(self)
        self.player = player

        self.set_range(0, 1)
        self.set_draw_value(False)
        self.set_update_policy(gtk.UPDATE_CONTINUOUS)

        self.connect("adjust-bounds", self.on_adjust_bounds)
        self.connect("button-press-event", self.on_button_press_event)
        self.connect("button-release-event", self.on_button_release_event)
        self.player.connect("notify::playtime", self.update)
        self.player.connect("notify::duration", self.update)

    def on_adjust_bounds(self, widget, value):
        if self.get_update_policy() == gtk.UPDATE_DISCONTINUOUS:
            return
        time = int(value * self.player.get_property("duration"))
        self.player.client.playback_seek_ms(time)

    def update(self, obj, property):
        if self.get_update_policy() == gtk.UPDATE_DISCONTINUOUS:
            return

        duration = self.player.get_property("duration")

        if duration > 0:
            self.set_sensitive(True)
            value    = self.player.get_property("playtime")
            new_value = value / float(duration)
            if abs(new_value - self.get_value()) > 0.001:
                self.set_value(new_value)
        else:
            self.set_sensitive(False)
            self.set_value(0)

    def on_button_press_event(self, widget, event):
        self.set_update_policy(gtk.UPDATE_DISCONTINUOUS)

    def on_button_release_event(self, widget, event):
        self.set_update_policy(gtk.UPDATE_CONTINUOUS)


class Statusbar(gtk.Statusbar):
    """
    Shows the current status of the daemon - playing, paused or stopped.
    """
    def __init__(self, player):
        gtk.Statusbar.__init__(self)
        self.player = player

        # Status subscription
        self.handler_id = self.player.connect("notify::status", self.on_status_notify)

        # Initial update
        self.player.notify("status")

    def on_status_notify(self, obj, property):
        value = self.player.get_property("status")
        self.update_statusbar(value)

    def update_statusbar(self, value):
        string = ""
        if value  == xmmsclient.PLAYBACK_STATUS_PLAY:
            try:
                bitrate = int(self.player.get_property("metadata").get(("server", "bitrate"))) / 1000
                string = "Playing, %s kbps" % (bitrate)
            except:
                string = "Playing"
        elif value == xmmsclient.PLAYBACK_STATUS_PAUSE:
            string = "Paused"
        elif value == xmmsclient.PLAYBACK_STATUS_STOP:
            string = "Stopped"
        self.pop(1)
        self.push(1, string)

    def destructor(self):
        self.player.disconnect(self.handler_id)
