"""
Copyright (C) 2005 Magnus Bjernstad <bjernstad@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import gtk, gtk.glade, os, functions, song_tree_view, xml.sax.saxutils, urllib, gobject

ENTITY_TYPE_FILE = 0
ENTITY_TYPE_DIR = 1
ENTITY_TYPE_PARENT = 2

class Library(gtk.VBox):
    def __init__(self, player):
        gtk.VBox.__init__(self)
        self.player = player
        self.song_view = song_tree_view.SongView(player, "library")

        glade_file = functions.glade_file("library")
        self.gxml_menu = gtk.glade.XML(glade_file, "lib_popup")
        self.gxml_menu.signal_autoconnect(self)

        # Event handlers
        self.song_view.view.connect("row-activated", self.on_row_activated)
        self.song_view.view.connect("button-press-event", self.on_button_press)
        self.song_view.view.connect("button-release-event", self.on_button_release)
        self.song_view.view.connect("popup-menu", self.on_popup_menu)

        self.model = gtk.ListStore(object, int)
        self.song_view.view.set_model(self.model)

        self.cwd_label = gtk.Label()
        self.cwd_label.set_alignment(0, 4)
        self.scrolled = gtk.ScrolledWindow()
        
        self.cell_pixbuf = gtk.CellRendererPixbuf()
        self.cell_text = gtk.CellRendererText()

        # Pack widgets
        self.pack_start(self.cwd_label, False, False, 3)
        self.pack_start(self.song_view)

        self.browse_state = BrowseState(player)
        self.browse_state.connect("changed", self.update_path_label)
        self.browse_state.connect("changed", self.on_browse_state_changed)
        self.browse_state.notify()

    def update_path_label(self, source, fields, values):
        escape = xml.sax.saxutils.escape
        fields = map(escape, fields)
        values = map(escape, values)

        fs = [f.title() + " = " for f in fields]
        label = " || ".join([a[0] + a[1] for a in zip(fs,
            ["<i>%s</i>" % (v,) for v in values])])
        if len(values) < len(fields):
            if label != "":
                label += " || "
            label += fields[len(values)].title() + " = <b>?</b>"
        self.cwd_label.set_markup(label)

    def show_menu(self, button=0, time=0):
        w = self.gxml_menu.get_widget
        selected = self.song_view.view.get_selection().count_selected_rows() > 0
        w("menu_play").set_sensitive(selected)
        w("menu_add").set_sensitive(selected)
        w("lib_popup").popup(None, None, None, button, time)

    def make_sql_query(self, fields, values):
        values = [v.replace("'", "''") for v in values]
        values = map(self.__quote_strings, values)

        nv = len(values)
        nf = len(fields)
        
        q = "SELECT m1.id, "
        q += "m%d.value" % (nv+1,)

        q += " FROM "
        q += ", ".join(["media m%d" % (i,) for i in range(1,nv+2)])

        q += " WHERE "
        q += " AND ".join(["m%d.id=m1.id" % (i,) for i in range(2,nv+2)])

        if nv > 0:
            q += " AND "
        q += " AND ".join(["m%d.key='%s'" % (i, v) for (i,v) in zip(range(1,nv+2), fields + ["url"])])

        if nv > 0:
            q += " AND "
        q += " AND ".join(["m%d.value=%s" % (i, v) for (i,v) in zip(range(1,nv+2), values)])

        q += " GROUP BY (m%i.value)" % (nv+1,)
        return q

    def __quote_strings(self, item):
        if type(item) == unicode:
            return "'%s'" % (item,)
        else:
            return str(item)


    def on_browse_state_changed(self, source, fields, values):
        escape = xml.sax.saxutils.escape
        fields = map(escape, fields)
        values = map(escape, values)

        self.model.clear()
        query = self.make_sql_query(fields, values)

        self.song_view.view.set_model(None)

        if len(values) == 0:
            self.player.client.medialib_select(query, self.non_leaf_query_cb)
        else:
            self.model.append(({"url" : os.pardir}, ENTITY_TYPE_PARENT))
            if len(values) < len(fields):
                # Still not at a leaf node
                self.player.client.medialib_select(query, self.non_leaf_query_cb)
            else:
                self.player.client.medialib_select(query, self.leaf_query_cb)


    def non_leaf_query_cb(self, res):
        res = res.value()
        if res == None:
            # No entries. Just ignore it.
            return
        for dict in res:
            try:
                self.model.append((dict, ENTITY_TYPE_DIR))
            except KeyError, e:
                print e
            except TypeError, e:
                print e
        functions.queue_function(self.player, self.song_view.view.set_model, self.model)

    def leaf_query_cb(self, res):
        res = res.value();
        if res == None:
            # No entries. Just ignore it.
            return
        for dict in res:
            self.player.client.medialib_get_info(int(dict["id"]), self.leaf_query_insert_cb)
        functions.queue_function(self.player, self.song_view.view.set_model, self.model)
        self.song_view.fill_end()

    def leaf_query_insert_cb(self, res):
        info = res.value()
        self.model.append((info, ENTITY_TYPE_FILE))

    def activate_row(self, path):
        iter = self.model.get_iter(path)
        dict = self.model.get_value(iter, 0)
        browse_type = self.model.get_value(iter, 1)

        if browse_type == ENTITY_TYPE_DIR:
            self.browse_state.push(dict["value"])

        elif browse_type == ENTITY_TYPE_PARENT:
            self.browse_state.pop()

        elif browse_type == ENTITY_TYPE_FILE:
            self.gxml_menu.get_widget("menu_add").activate()

    def add_selected(self):
        self.__add_selected()

    def play_selected(self):
        self.player.client.playlist_clear()
        self.__add_selected(self.start_playback)

    def start_playback(self):
        self.player.client.playlist_set_next(0)
        self.player.client.playback_tickle()
        self.player.client.playback_start()

    def __add_selected(self, callback=None):
        self.callback = callback
        model, paths = self.song_view.view.get_selection().get_selected_rows()

        # TODO: Find a less ugly way to detect when everything has been added.
        self.done = False
        for path in paths:
            if path == paths[-1]:
                self.done = True
            entity_type = self.model[path][1]
            selected = self.model[path][0]

            if entity_type == ENTITY_TYPE_FILE:
                id = selected[("server", "id")]
                self.player.client.playlist_add_id(id)
                if self.done == True:
                    functions.queue_function(self.player, self.callback)
            elif entity_type == ENTITY_TYPE_DIR:
                value = selected["value"]
                values = self.browse_state.values + [value]
                fields = self.browse_state.fields[0:len(values)]
                q = self.make_sql_query(fields, values)
                self.player.client.medialib_select(q, self.add_ids_query_cb)

    def add_ids_query_cb(self, res):
        res = res.value()
        if res == None:
            return # No entries. Just ignore it.
        for dict in res:
            self.player.client.playlist_add_id(dict["id"])
        if self.done:
            functions.queue_function(self.player, self.callback)


    ################
    # Event handlers
    ################
    def on_row_activated(self, treeview, path, view_column):
        self.activate_row(path)

    def on_button_press(self, widget, event):
        if event.button == 3:
            return self.song_view.view.get_selection().count_selected_rows() > 1

    def on_button_release(self, widget, event):
        if event.button == 3:
            self.show_menu(event.button, event.time)
        elif event.button == 2:
            try:
                if self.model[0][1] == ENTITY_TYPE_PARENT:
                    self.activate_row(0)

            except IndexError:
                # Model is empty!
                print "Warning: Model is empty."
        return True

    def on_popup_menu(self, widget):
        self.show_menu()
        
    ################
    # Menu callbacks
    ################
    def on_menu_play(self, menuitem):
        self.play_selected()

    def on_menu_add(self, menuitem):
        self.add_selected()

    def on_menu_rehash_all(self, menuitem):
        self.player.client.medialib_rehash()

    def on_menu_import(self, menuitem):
        fc = gtk.FileChooserDialog("Directory to import",
                None,
                gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
                (gtk.STOCK_OK, gtk.RESPONSE_OK,
                    gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
        response = fc.run()

        if response == gtk.RESPONSE_OK:
            uri = fc.get_uri()
            uri = urllib.unquote(uri)
            if uri.startswith("file://"):
                uri = uri[len("file://"):]
            self.player.client.medialib_path_import(uri)

        fc.destroy()

    def on_info(self, widget):
        model, rows = self.song_view.view.get_selection().get_selected_rows()
        if len(rows) > 0:
            row = rows[0]
            if self.model[row][1] == ENTITY_TYPE_FILE:
                info_map = self.model[row][0]
                song_tree_view.TagInfo(info_map)

class BrowseState(gobject.GObject):
    """
    Keeps track of the current position in the media library.
    The ones interested can connect to the 'changed' signal, which is emitted
    when a new place in the library hierarchy has been selected.
    """
    __gsignals__ = {
        "changed" : (gobject.SIGNAL_RUN_LAST,
                gobject.TYPE_NONE,
                (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT))
    }

    def __init__(self, player):
        gobject.GObject.__init__(self)
        self.values = []
        self.fields = []

        player.settings.add_cb("library", "fields", self.fields_cb)
        player.settings.emit("library", "fields")

    def fields_cb(self, section, option, value):
        self.fields = map (str.lower, value.split(","))
        self.values = []
        self.notify()

    def push(self, value):
        self.values.append(value)
        self.notify()

    def pop(self):
        self.values.pop()
        self.notify()

    def notify(self):
        self.emit("changed", self.fields, self.values)

if gtk.check_version(2, 8, 0):
    gobject.type_register(BrowseState)
