"""
Copyright (C) 2005 Magnus Bjernstad <bjernstad@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import os, gtk, gtk.glade, xmmsclient, song_tree_view, functions, gobject



class Playlist(gtk.VBox):
    SORT_TYPES = ["album", "title", "url", "bitrate", "artist", "tracknr", "date", "mime", "genre"]
    def __init__(self, player):
        gtk.VBox.__init__(self)
        self.player = player

        # Glade menu
        self.glade_file = functions.glade_file("playlist")
        self.gxml_menu = gtk.glade.XML(self.glade_file, "pl_popup")
        self.gxml_menu.signal_autoconnect(self)
        w = self.gxml_menu.get_widget

        # Create widgets
        self.song_view = song_tree_view.SongView(player, "playlist")

        v = self.song_view.view
        self.model = gtk.ListStore(object, bool) # (info_hash, is_playing)
        self.filter = self.model.filter_new()
        self.flt = Filter(player, self.filter)
        menu = gtk.Menu()
        for value in self.SORT_TYPES:
            item = gtk.MenuItem(value.title())
            menu.append(item)
            item.connect("activate", self.on_menu_sort_by)
        menu.show_all()

        # Init accelerators
        accel = gtk.AccelGroup()
        w("save").add_accelerator("activate", accel, ord("s"), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
        w("info").add_accelerator("activate", accel, ord("i"), gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
        player.window.add_accel_group(accel)

        # Setup drag and drop
        targets = [('STRING', gtk.TARGET_SAME_WIDGET, 0)]
        v.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, targets, gtk.gdk.ACTION_MOVE)
        v.enable_model_drag_dest(targets, gtk.gdk.ACTION_MOVE)

        # Pack widgets
        w("sort").set_submenu(menu)
        self.pack_start(self.song_view)

        # Set properties
        self.marked = -1
        self.in_playlist = False
        v.set_model(self.filter)
        self.song_view.set_clickable(True)

        # Callbacks
        player.settings.add_cb("playlist", "show_filter", self.show_filter_cb)
        player.settings.add_cb("playlist", "color", self.playlist_color_cb)

        # Event handlers
        player.connect("notify::position", self.on_position_changed)
        player.connect("playlist-changed", self.on_playlist_changed)

        v.connect("row-activated", self.on_row_activated)
        v.connect("button-press-event", self.on_button_press)
        v.connect("button-release-event", self.on_button_release)
        v.connect("key-press-event", self.on_key_press)
        v.connect_after("columns-changed", self.on_columns_changed)
        v.connect("drag-drop", self.on_drag_drop)
        for col in v.get_columns():
            col.connect("clicked", self.on_column_clicked)

        # Initial update
        self.reload_playlist()
        v.emit("columns-changed")
        player.notify("position")
        player.settings.emit("playlist", "show_filter")
        player.settings.emit("playlist", "color")

    def reload_playlist(self):
        self.song_view.fill_begin()
        self.model.clear()
        self.player.client.playlist_list(self.reload_playlist_list_cb)

    def update_marked(self):
        """
        Marks the currently playing position in the playlist.
        """
        try:
            new_pos = self.player.get_property("position")
        except xmmsclient.XMMSError:
            # Probably no current entry. Do nothing.
            return

        if self.marked >= 0:
            try:
                self.model[self.marked][1] = False
            except IndexError:
                # Old marked position has been removed. Just ignore it.
                pass
        if new_pos >= 0 and self.in_playlist:
            try:
                self.model[new_pos][1] = True
                self.marked = new_pos
            except IndexError:
                pass

    def show_menu(self, button=0, time=0):
        selected = self.song_view.view.get_selection().count_selected_rows() > 0
        self.gxml_menu.get_widget("delete").set_sensitive(selected)
        self.gxml_menu.get_widget("crop").set_sensitive(selected)
        self.gxml_menu.get_widget("pl_popup").popup(None, None, None, button, time)
    
    def center(self, force=False):
        if self.player.settings.get("playlist", "auto_center") == "0" and force==False:
            return
        try:
            pos = self.player.get_property("position")
        except xmmsclient.XMMSError:
            # Probably no current entry
            return
        if pos < 0 or pos >= len(self.filter):
            return
        flt_pos = self.filter.convert_child_path_to_path(pos)
        if flt_pos != None:
            try:
                self.song_view.view.scroll_to_cell(flt_pos, use_align=True, row_align=0.5)
            except TypeError:
                pass

    def destructor(self):
        self.song_view.destructor()
        self.player.settings.remove_cb("playlist", "show_filter", self.show_filter_cb)
        self.player.settings.remove_cb("playlist", "color", self.playlist_color_cb)

    ######################
    # Client lib callbacks
    ######################
    def add_cb(self, res):
        info = res.value()
        self.model.append((info, False))

    def reload_playlist_list_cb(self, res):
        ids = res.value()

        self.song_view.view.set_model(None)

        try:
            for id in ids:
                self.player.client.medialib_get_info(id, self.reload_medialib_get_info_cb)
        except TypeError:
            pass
        functions.queue_function(self.player, self.song_view.view.set_model, self.filter)
        self.song_view.fill_end()
        functions.queue_function(self.player, self.update_marked)

    def reload_medialib_get_info_cb(self, res):
        info = res.value()
        try:
            self.model.append((info, False))
        except TypeError:
            print >>sys.stderr, "Playlist: Warning: Couldn't append item."

    ####################
    # Settings callbacks
    ####################

    def show_filter_cb(self, section, option, value):
        if value == "1":
            if not self.flt.get_parent():
                self.pack_start(self.flt, False, False)
        else:
            self.flt.entry.set_text("")
            self.remove(self.flt)

    def playlist_color_cb(self, section, option, value):
        self.song_view.cell.set_property("background", value)
        self.song_view.view.queue_draw()

    #################
    # Event handlers
    #################

    def on_row_activated(self, treeview, path, view_column):
        selected_pos = path[0]
        new_pos = self.filter.convert_path_to_child_path(selected_pos)
        self.player.client.playlist_set_next(new_pos[0])
        self.player.client.playback_tickle()
        self.player.client.playback_start()

    def on_button_press(self, widget, event):
        if event.button == 3:
            return widget.get_selection().count_selected_rows() > 1

    def on_button_release(self, widget, event):
        if event.button == 3:
            self.show_menu(event.button, event.time)

    def on_key_press(self, widget, event):
        if event.keyval == gtk.gdk.keyval_from_name("c") and event.state & gtk.gdk.CONTROL_MASK:
            self.center(force=True)
            return True
        elif event.keyval == gtk.gdk.keyval_from_name("Delete"):
            self.gxml_menu.get_widget("delete").activate()
            return True

    def on_columns_changed(self, treeview):
        for col in self.song_view.view.get_columns():
            col.add_attribute(self.song_view.cell, "background-set", 1)

    def on_drag_drop(self, widget, drag_context, x, y, timestamp):
        widget.emit_stop_by_name("drag-drop")
        model, rows = widget.get_selection().get_selected_rows()
        if len(rows) < 1:
            return
        rows = [row[0] for row in rows]
        drop_info = widget.get_dest_row_at_pos(0, y)

        if drop_info:
            new_pos, position = drop_info
            new_pos = new_pos[0]
            if new_pos > rows[0]:
                if position == gtk.TREE_VIEW_DROP_BEFORE:
                    new_pos -= 1
            else:
                if position == gtk.TREE_VIEW_DROP_AFTER:
                    new_pos += 1
        else:
            new_pos = len(model)-1

        if new_pos < rows[0]:
            rows.reverse()

        # TODO: Multiple rows doesn't work correctly. Just move the first in
        # the meantime.
        #for row in rows:
        self.player.client.playlist_move(rows[0], new_pos)

    def on_position_changed(self, obj, property):
        self.update_marked()
        self.in_playlist = True
        self.center()

    def on_playlist_changed(self, obj, value):
        event = int(value["type"])

        if event == xmmsclient.PLAYLIST_CHANGED_ADD:
            pos = int(value["position"])
            id = int(value["id"])
            self.player.client.medialib_get_info(id, self.add_cb)

        elif event == xmmsclient.PLAYLIST_CHANGED_CLEAR:
            self.model.clear()

        elif event == xmmsclient.PLAYLIST_CHANGED_MOVE:
            old = value["position"]
            new = value["newposition"]
            old_iter = self.model[old].iter
            new_iter = self.model[new].iter
            if old > new:
                self.model.move_before(old_iter, new_iter)
            else:
                self.model.move_after(old_iter, new_iter)

        elif event == xmmsclient.PLAYLIST_CHANGED_REMOVE:
            pos = int(value["position"])
            if pos == self.marked:
                try:
                    self.in_playlist = False
                    self.model[self.marked][1] = False
                    self.marked = -1
                except IndexError:
                    pass
            try:
                iter = self.model.get_iter(pos)
            except ValueError:
                # Not a valid iter
                self.reload_playlist()
                return
            self.model.remove(iter)
            selection = self.song_view.view.get_selection()
            selection.unselect_all()
            selection.select_path(pos)
            if not selection.path_is_selected(pos):
                pos -= 1
                if pos >= 0:
                    selection.select_path(pos)

        elif event == xmmsclient.PLAYLIST_CHANGED_SHUFFLE:
            self.reload_playlist()

        elif event == xmmsclient.PLAYLIST_CHANGED_SORT:
            self.reload_playlist()

    def on_column_clicked(self, column):
        try:
            sort_type = column.get_title().lower()
            self.player.client.playlist_sort(sort_type)
        except KeyError:
            pass


    #####################
    # Menu event handlers
    #####################

    def on_menu_add(self, widget):
        md = gtk.MessageDialog(None,
                gtk.DIALOG_DESTROY_WITH_PARENT,
                gtk.MESSAGE_QUESTION,
                gtk.BUTTONS_OK_CANCEL)
        md.set_markup('<span weight="bold" size="larger">Add stream</span>')
        entry = gtk.Entry()
        md.vbox.pack_start(entry)
        md.vbox.show_all()
        result = md.run()

        if result == gtk.RESPONSE_OK:
            url = entry.get_text()
            if url != "":
                self.player.client.playlist_add(url)
        md.destroy()

    def on_menu_save(self, widget):
        md = gtk.MessageDialog(None,
                gtk.DIALOG_DESTROY_WITH_PARENT,
                gtk.MESSAGE_QUESTION,
                gtk.BUTTONS_OK_CANCEL)
        md.set_markup('<span weight="bold" size="larger">Save new playlist</span>')
        entry = gtk.Entry()
        md.vbox.pack_start(entry)
        md.vbox.show_all()
        result = md.run()
        md.destroy()
        if result == gtk.RESPONSE_OK:
            save_name = entry.get_text()
            if save_name != "":
                self.player.client.medialib_playlist_save_current(save_name)

    def on_menu_shuffle(self, widget):
        self.player.client.playlist_shuffle()
        
    def on_menu_refresh_metadata(self, widget):
        self.reload_playlist()

    def on_menu_delete(self, widget):
        model, rows = self.song_view.view.get_selection().get_selected_rows()
        rows.reverse()
        for row in rows:
            res = self.player.client.playlist_remove(row[0])

    def on_menu_crop(self, widget):
        model, sel_paths = self.song_view.view.get_selection().get_selected_rows()
        sel_paths = [self.filter.convert_path_to_child_path(x) for x in sel_paths]
        all_paths = [x.path for x in self.model]
        to_del = filter(lambda x: x not in sel_paths, all_paths)
        to_del.reverse()
        for id in to_del:
            self.player.client.playlist_remove(id[0])

    def on_menu_clear(self, widget):
        self.player.client.playlist_clear()

    def on_menu_sort_by(self, menuitem):
        sort_type = menuitem.child.get_text().lower()
        self.player.client.playlist_sort(sort_type)

    def on_info(self, widget):
        model, rows = self.song_view.view.get_selection().get_selected_rows()
        if len(rows) > 0:
            row = rows[0]
            info_map = self.model[row][0]
            song_tree_view.TagInfo(info_map)
            
    
class Filter(gtk.HBox):
    FILTER_FIELDS = ["album", "title", "url", "bitrate", "artist", "tracknr", "date", "mime", "genre"]

    def __init__(self, player, tree_model_filter):
        gtk.HBox.__init__(self)
        self.player = player
        self.tree_model_filter = tree_model_filter

        self.entry = gtk.Entry()
        self.combobox = gtk.combo_box_new_text()

        for field in self.FILTER_FIELDS:
            self.combobox.append_text(field.title())

        # Restore last filter field
        active_field = self.player.settings.get("playlist", "filter_field")
        try:
            active_index = self.FILTER_FIELDS.index(active_field)
        except ValueError:
            active_index = 0
        self.combobox.set_active(active_index)

        # Set filter functions
        self.tree_model_filter.set_visible_func(self.visible_func)

        # Signal handlers
        self.entry.connect("changed", self.refilter)
        self.combobox.connect("changed", self.refilter)

        # Pack widgets
        label = gtk.Label("F_ilter: ")
        label.set_mnemonic_widget(self.entry)
        label.set_use_underline(True)
        self.pack_start(label, False, False)
        self.pack_start(self.entry)
        self.pack_start(self.combobox, False, False)


    def visible_func(self, model, iter):
        what = self.entry.get_text().lower()
        if what == "":
            return True
        property = self.combobox.get_active_text().lower()
        info_hash = model.get_value(iter, 0)
        try:
            where = info_hash[property]
        except KeyError:
            return False
        return str(where).lower().find(what) >= 0

    def refilter(self, widget):
        field = self.combobox.get_active_text().lower()
        self.player.settings.set("playlist", "filter_field", field)
        self.tree_model_filter.refilter()
