"""
Copyright (C) 2005 Magnus Bjernstad <bjernstad@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import gtk, gtk.glade, os, functions, song_tree_view, xmmsclient, sys

class Playlists(gtk.VBox):
    """
    Manages a list of all the playlists stored in xmms' database.
    """
    def __init__(self, player):
        gtk.VBox.__init__(self)
        self.player = player
        self.song_view = song_tree_view.SongView(player, "playlists")

        glade_file = functions.glade_file("playlists")
        self.gxml_menu = gtk.glade.XML(glade_file, "popup")
        self.gxml_menu.signal_autoconnect(self)

        self.song_view.view.connect("row-activated", self.on_row_activated)
        self.song_view.view.connect("button-press-event", self.on_button_press)
        self.song_view.view.connect("button-release-event", self.on_button_release)
        self.song_view.view.connect("row-expanded", self.on_row_expanded)

        self.model = gtk.TreeStore(object)
        self.song_view.view.set_model(self.model)

        self.pack_start(self.song_view)

        # Initial update
        self.reload_playlists()

    def reload_playlists(self):
        """
        Forces a reload of all playlists.
        """
        self.model.clear()
        self.player.client.medialib_select("SELECT * from Playlist", self.playlists_list_cb)

    def show_menu(self, button=0, time=0):
        """
        Popup the context menu.
        """
        w = self.gxml_menu.get_widget
        selected = self.song_view.view.get_selection().count_selected_rows() > 0
        w("play").set_sensitive(selected)
        w("add").set_sensitive(selected)
        w("delete").set_sensitive(selected)
        w("popup").popup(None, None, None, button, time)

    def destructor(self):
        self.song_view.destructor()

    def add_selected(self):
        self.__add_selected()

    def play_selected(self):
        self.player.client.playlist_clear()
        self.__add_selected(self.start_playback)

    def start_playback(self):
        self.player.client.playlist_set_next(0)
        self.player.client.playback_tickle()
        self.player.client.playback_start()

    def __add_selected(self, callback=None):
        model, paths = self.song_view.view.get_selection().get_selected_rows()

        for path in paths:
            if len(path) == 1:
                pl_name = self.model[path][0]["name"]
                pl_name = pl_name.replace("'", "''")
                self.player.client.medialib_playlist_load(pl_name)
            elif len(path) == 2:
                id = self.model[path][0]["id"]
                self.player.client.playlist_add_id(id)

        functions.queue_function(self.player, callback)

    ##########################
    # Client library callbacks
    ##########################

    def playlists_list_cb(self, res):
        dicts = res.value()

        if dicts == None:
            # Zero playlists
            return
        for dict in dicts:
            # Don't show hidden playlists
            if dict["name"][0] == "_":
                continue
            iter = self.model.append(None, (dict,))
            self.model.append(iter, (None,))

    def playlist_export_cb(self, res):
        text = res.value()
        file = open(self.save_path, "w")
        try:
            try:
                file.write(text)
            finally:
                file.close()
        except:
            print >>sys.stderr, "Playlists: Failed to export playlist."


    ################
    # Event handlers
    ################
    def on_row_activated(self, treeview, path, view_column):
        """
        Double clicking a row is the same as choosing 'Play' from the context
        menu.
        """
        self.gxml_menu.get_widget("play").activate()

    def on_button_press(self, widget, event):
        """
        Prevend deselection of multiple rows at mouse button press.
        """
        if event.button == 3:
            return self.song_view.view.get_selection().count_selected_rows() > 1

    def on_button_release(self, widget, event):
        """
        Show the context menu.
        """
        if event.button == 3:
            self.show_menu(event.button, event.time)

    def on_row_expanded(self, view, iter, path):
        model = view.get_model()
        child = model.iter_children(iter)
        dict = model.get_value(iter, 0)
        if model.get_value(child, 0) is None:
            model.remove(child)
            query = "SELECT * FROM playlistentries WHERE playlist_id=%s ORDER BY pos" % (dict["id"])
            self.player.client.medialib_select(query, RowExpandedHandler(self, path))
            view.expand_row(path, False)
    

    ################
    # Menu callbacks
    ################
    def on_play_activate(self, widget):
        """
        Replace current playlist with selected playlists.
        """
        self.play_selected()

    def on_add_activate(self, widget):
        """
        Add all selected playlists to the playlist.
        """
        self.add_selected()

    def on_refresh_activate(self, widget):
        """
        Update the playlists list.
        """
        self.reload_playlists()

    def on_delete_activate(self, widget):
        """
        If a playlist is selected, delete it after confirmation.
        If a playlist entry is selected, delete it from the playlist.
        """
        model, paths = self.song_view.view.get_selection().get_selected_rows()

        # Loop through all selected rows
        for path in paths:
            name = model[path[0]][0]["name"]
            name = name.replace("'", "''")

            if len(path) == 1: # Playlist
                md = gtk.MessageDialog(None,
                        gtk.DIALOG_DESTROY_WITH_PARENT,
                        gtk.MESSAGE_QUESTION,
                        gtk.BUTTONS_YES_NO,
                        "Are you sure you want to delete '" + name + "'?")
                result = md.run()
                if result == gtk.RESPONSE_YES:
                    self.player.client.medialib_playlist_remove(name)
                md.destroy()

            else: # Playlist entry
                # TODO: Identify a playlist entry uniquely. The current way
                # doesn't work properly if there are duplicates of one entry in
                # the same playlist.
                pl_id = model[path[0]][0]["id"]
                song_id = model[path][0]["id"]
                query = "DELETE FROM playlistentries WHERE playlist_id=%d AND entry='mlib://%d' LIMIT 1" % (pl_id, song_id)
                self.player.client.medialib_select(query)
        self.reload_playlists()

    def on_export_activate(self, widget):
        model, paths = self.song_view.view.get_selection().get_selected_rows()

        # Loop through all selected rows
        for path in paths:
            name = model[path[0]][0]["name"]

            if len(path) == 1: # Playlist
                fc = gtk.FileChooserDialog("Export playlist '%s'" % (name),
                        None, gtk.FILE_CHOOSER_ACTION_SAVE,
                        (gtk.STOCK_OK, gtk.RESPONSE_OK,
                            gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))

                filter = gtk.FileFilter()
                filter.set_name("M3U playlists")
                filter.add_pattern("*.m3u")
                fc.add_filter(filter)

                filter = gtk.FileFilter()
                filter.set_name("PLS playlists")
                filter.add_pattern("*.pls")
                fc.add_filter(filter)

                filter = gtk.FileFilter()
                filter.set_name("HTML playlists")
                filter.add_pattern("*.html")
                fc.add_filter(filter)

                filter = gtk.FileFilter()
                filter.set_name("All files")
                filter.add_pattern("*")
                fc.add_filter(filter)

                while 1:
                    response = fc.run()
                    if response == gtk.RESPONSE_OK:
                        self.save_path = fc.get_filename()
                        ext = os.path.splitext(self.save_path)[1]
                        overwrite = True
                        
                        if ext == ".m3u":
                            mime = "audio/mpegurl"
                        elif ext == ".pls":
                            mime = "audio/x-scpls"
                        elif ext == ".html":
                            mime = "text/html"
                        else:
                            mime = "audio/mpegurl"


                        if  os.path.exists(self.save_path):
                            md = gtk.MessageDialog(None,
                                    gtk.DIALOG_DESTROY_WITH_PARENT,
                                    gtk.MESSAGE_QUESTION,
                                    gtk.BUTTONS_NONE)
                            md.set_markup('<span weight="bold" size="larger">File already exists</span>')
                            md.add_buttons("Overwrite", gtk.RESPONSE_OK, gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
                            result = md.run()
                            if result == gtk.RESPONSE_CANCEL:
                                overwrite = False
                            md.destroy()

                        if overwrite:
                            res = self.player.client.medialib_playlist_export(name, mime, self.playlist_export_cb)
                            break

                    elif response == gtk.RESPONSE_CANCEL:
                        break

                fc.destroy()

    def on_info(self, widget):
        model, rows = self.song_view.view.get_selection().get_selected_rows()
        if len(rows) > 0:
            row = rows[0]
            if len(row) == 2:
                info_map = self.model[row][0]
                song_tree_view.TagInfo(info_map)


class RowExpandedHandler:
    def __init__(self, caller, path):
        self.caller = caller
        self.path = path

    def __call__(self, res):
        dict_list = res.value()
        get_info = self.caller.player.client.medialib_get_info
        for dict in dict_list:
            entry = dict["entry"]
            if entry.find("mlib://") == 0:
                id = int(entry[len("mlib://"):])
                get_info(id, self.info_cb)

    def info_cb(self, res):
        info = res.value()
        iter = self.caller.model.get_iter(self.path)
        self.caller.model.append(iter, (info,))
        self.caller.song_view.view.expand_row(self.path, False)


