"""
Copyright (C) 2005 Magnus Bjernstad <bjernstad@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import xml.sax.saxutils, ConfigParser, os, sys

class Settings:
    """
    Keep track of all the current configuration values. Callback functions can
    be used to recieve notification when a config value has changed.

    The default values read from a file "defaults.conf" is overridden by the
    user config file in ~/.xmms2/clients/defaults.conf.
    """
    def __init__(self):
        """
        Read old config values.
        """
        self.config = ConfigParser.ConfigParser()
        found_defaults = False
        defaults_files = ["defaults.conf",
                          "/usr/share/shellac/defaults.conf",
                          "/usr/local/share/shellac/defaults.conf"]
        for f in defaults_files:
            try:
                self.config.readfp(open(f))
                found_defaults = True
                break
            except:
                pass
        if not found_defaults:
            print >>sys.stderr, "Settings: Could not load default configuration"
            sys.exit()
        user_config = os.path.expanduser("~/.xmms2/clients/shellac.conf")
        self.config.read([user_config])
        self.callbacks = {}

    def add_cb(self, section, option, callback):
        """
        Add a callback function to be called when a specified option is
        changed.
        It must be removed with L{remove_cb}

        @param section: The section to monitor.
        @param option: The option to monitor.
        @param callback: The function to call upon changes. The signature must be:
            def callback(section, option, value)
        """
        key = "%s.%s" % (section, option)
        if self.callbacks.has_key(key):
            self.callbacks[key].append(callback)
        else:
            self.callbacks[key] = [callback]

    def remove_cb(self, section, option, callback):
        """
        Stop a callback function to be called when a specified option is
        changed.

        @param section: Remove the callback from this section.
        @param option: Remove the callback from this option.
        @param callback: The callback function to remove.
        """
        key = "%s.%s" % (section, option)
        for func in self.callbacks[key]:
            if func == callback:
                self.callbacks[key].remove(callback)

    def get(self, section, option):
        """
        Retrieve a configuration value.

        @param: The section to retrieve the config option from.
        @param: The option to return.
        """
        return self.config.get(section, option)

    def set(self, section, option, value):
        """
        Set a config option to a value. Notify all listeners.

        @param section: The section in which the option is located.
        @param option: The option to set.
        @param value: The new value.
        """
        self.config.set(section, option, value)
        try:
            for func in self.callbacks["%s.%s" % (section, option)]:
                func(section, option, value)
        except KeyError:
            # Nobody is subscribing to this key
            pass

    def emit(self, section, option):
        """
        Notify all listeners even if nothing has changed.

        @param section: The section in which the option is located.
        @param option: The option to set.
        """
        value = self.get(section, option)
        self.set(section, option, value)

    def write(self):
        """
        Write the current configuraton to the user's configuration file.
        """
        config_file = os.path.expanduser("~/.xmms2/clients/shellac.conf")
        fo = file(config_file, "w")
        try:
            self.config.write(fo)
        finally:
            fo.close()

