"""
Copyright (C) 2005 Magnus Bjernstad <bjernstad@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import gtk, gtk.glade, xml.sax.saxutils, os, functions, re, xmmsclient

LIBRARY_FIELDS = ["Artist", "Album", "Title", "Genre",
           "Date", "Tracknr", "Comment", "Duration",
           "Bitrate"]
GUI_WIDGETS = ["Previous", "Play", "Stop", "Next", "Repeat",
               "Time", "Seek bar", "Volume", "Preferences"]
COLUMNS = ["Url", "Artist", "Album", "Title", "Genre",
           "Date", "Tracknr", "Comment", "Duration",
           "Bitrate", "Custom"]


class SettingsEditor:
    def __init__(self, player):
        self.settings = player.settings

        self.glade_file = functions.glade_file("settings")
        gxml = gtk.glade.XML(self.glade_file, "preferences")
        gxml.signal_autoconnect(self)

        w = gxml.get_widget

        # General
        EntryOptionEditor(player, "global", "ipcpath", w("ipc"))

        # Gui
        ToggleListOptionEditor(player, "gui", "row_1", w("gui_row_1_view"), GUI_WIDGETS)
        ToggleListOptionEditor(player, "gui", "row_2", w("gui_row_2_view"), GUI_WIDGETS)
        TextViewOptionEditor(player, "gui", "status_format", w("status_format"))
        BoolOptionEditor(player, "gui", "show_statusbar", w("show_statusbar"))
        EntryOptionEditor(player, "gui", "time_format", w("time_format"))

        # Playlist
        BoolOptionEditor(player, "playlist", "show_filter", w("show_filter"))
        ColorButtonOptionEditor(player, "playlist", "color", w("playlist_active_colorbutton"))
        view = w("playlist_format_view")
        ColumnsOptionEditor(player, "playlist", "columns", view, COLUMNS)
        TextViewOptionEditor(player, "playlist", "custom_format", w("playlist_custom_format"))
        BoolOptionEditor(player, "playlist", "rules_hint", w("playlist_rules_hint"))
        BoolOptionEditor(player, "playlist", "auto_center", w("auto_center"))

        # Library
        view = w("library_format_view")
        ToggleListOptionEditor(player, "library", "fields", view, LIBRARY_FIELDS)
        TextViewOptionEditor(player, "library", "custom_format", w("library_custom_format"))
        BoolOptionEditor(player, "library", "rules_hint", w("library_rules_hint"))

        # Find
        BoolOptionEditor(player, "find", "incremental", w("incremental_search"))
        view = w("find_format_view")
        ColumnsOptionEditor(player, "find", "columns", view, COLUMNS)
        TextViewOptionEditor(player, "find", "custom_format", w("find_custom_format"))
        BoolOptionEditor(player, "find", "rules_hint", w("find_rules_hint"))

        # Playlists
        view = w("playlists_format_view")
        TextViewOptionEditor(player, "playlists", "custom_format", w("playlists_custom_format"))
        BoolOptionEditor(player, "playlists", "rules_hint", w("playlists_rules_hint"))

        # Stream browser
        view = w("stream_browser_format_view")
        TextViewOptionEditor(player, "stream_browser", "custom_format", w("stream_browser_custom_format"))
        BoolOptionEditor(player, "stream_browser", "rules_hint", w("stream_browser_rules_hint"))

        # File browser
        view = w("file_browser_format_view")
        TextViewOptionEditor(player, "file_browser", "custom_format", w("file_browser_custom_format"))
        BoolOptionEditor(player, "file_browser", "rules_hint", w("file_browser_rules_hint"))
        EntryOptionEditor(player, "file_browser", "extensions", w("file_browser_extensions"))
        BoolOptionEditor(player, "file_browser", "remember_cwd", w("file_browser_remember_cwd"))
        
        # Output plugin
        OutputPluginSelector(player, self.glade_file, w("output_plugin"), w("output_plugin_options"))

        # Advanced
        scrolled = w("advanced_scrolled_window")
        view = AdvancedConfigView(player)
        scrolled.add(view)

        scrolled.show_all()

class OutputPluginSelector:
    def __init__(self, player, glade_file, combobox, options_button):
        self.player = player
        self.glade_file = glade_file
        self.combobox = combobox
        options_button.connect("clicked", self.on_output_plugin_options)

        # Find the currently used plugin
        self.player.client.configval_get("output.plugin", self.get_current_cb)

    def get_current_cb(self, res):
        self.current = res.value()

        # Now get all the possible plugins
        self.player.client.plugin_list(xmmsclient.PLUGIN_TYPE_OUTPUT, self.plugin_list_cb)

    def plugin_list_cb(self, res):
        plugins = [x["shortname"] for x in res.value()]
        plugins.sort()

        for plugin in plugins:
            self.combobox.append_text(plugin)
        try:
            active_index = plugins.index(self.current)
        except ValueError:
            active_index = -1
        self.combobox.set_active(active_index)

        self.combobox.connect("changed", self.on_changed)

    def on_changed(self, combobox):
        plugin = combobox.get_active_text()
        self.player.client.configval_set("output.plugin", plugin)

    def on_output_plugin_options(self, widget):
        plugin = self.combobox.get_active_text()
        if plugin == "alsa":
            OutputPluginOptionsAlsa(self.player, self.glade_file)
        elif plugin == "diskwrite":
            OutputPluginOptionsDiskwrite(self.player, self.glade_file)
        elif plugin == "oss":
            OutputPluginOptionsOss(self.player, self.glade_file)
        else:
            OutputPluginOptionsEmpty()


class OutputPluginOptionsEmpty(gtk.MessageDialog):
    def __init__(self):
        gtk.MessageDialog.__init__(self,
                flags=gtk.DIALOG_DESTROY_WITH_PARENT,
                buttons=gtk.BUTTONS_OK)
        self.set_title("Output plugin options")
        self.set_markup('<span weight="bold" size="larger">No options available.</span>')
        self.run()
        self.destroy()

class OutputPluginOptionsAlsa:
    def __init__(self, player, glade_file):
        self.player = player
        gxml = gtk.glade.XML(glade_file, "output_plugin_options_alsa")
        gxml.signal_autoconnect(self)
        self.w = gxml.get_widget
        player.client.configval_get("output.alsa.device", self.get_device_cb)
        player.client.configval_get("output.alsa.mixer", self.get_mixer_cb)
        player.client.configval_get("output.alsa.mixer_dev", self.get_mixer_dev_cb)


    def get_device_cb(self, res):
        self.w("device").child.set_text(res.value())
    def get_mixer_cb(self, res):
        self.w("mixer").child.set_text(res.value())
    def get_mixer_dev_cb(self, res):
        self.w("mixer_dev").child.set_text(res.value())

    def on_cancel(self, widget):
        self.w("output_plugin_options_alsa").destroy()

    def on_ok(self, widget):
        device = self.w("device").get_active_text()
        mixer = self.w("mixer").get_active_text()
        mixer_dev = self.w("mixer_dev").get_active_text()

        self.player.client.configval_set("output.alsa.device", device)
        self.player.client.configval_set("output.alsa.mixer", mixer)
        self.player.client.configval_set("output.alsa.mixer_dev", mixer_dev)

        self.w("output_plugin_options_alsa").destroy()

class OutputPluginOptionsOss:
    def __init__(self, player, glade_file):
        self.player = player
        gxml = gtk.glade.XML(glade_file, "output_plugin_options_oss")
        gxml.signal_autoconnect(self)
        self.w = gxml.get_widget
        player.client.configval_get("output.oss.device", self.get_device_cb)
        player.client.configval_get("output.oss.mixer", self.get_mixer_cb)


    def get_device_cb(self, res):
        self.w("device").child.set_text(res.value())

    def get_mixer_cb(self, res):
        self.w("mixer").child.set_text(res.value())

    def on_cancel(self, widget):
        self.w("output_plugin_options_oss").destroy()

    def on_ok(self, widget):
        device = self.w("device").get_active_text()
        mixer = self.w("mixer").get_active_text()

        self.player.client.configval_set("output.oss.device", device)
        self.player.client.configval_set("output.oss.mixer", mixer)

        self.w("output_plugin_options_oss").destroy()

class OutputPluginOptionsDiskwrite:
    def __init__(self, player, glade_file):
        self.player = player
        gxml = gtk.glade.XML(glade_file, "output_plugin_options_diskwrite")
        gxml.signal_autoconnect(self)
        self.w = gxml.get_widget
        player.client.configval_get("output.diskwrite.destination_directory", self.get_destination_directory_cb)

    def get_destination_directory_cb(self, res):
        self.w("directory").set_current_folder(res.value())

    def on_cancel(self, widget):
        self.w("output_plugin_options_diskwrite").destroy()

    def on_ok(self, widget):
        dir = self.w("directory").get_current_folder()
        self.player.client.configval_set("output.diskwrite.destination_directory", dir)
        self.w("output_plugin_options_diskwrite").destroy()


class AdvancedConfigView(gtk.TreeView):
    def __init__(self, player):
        gtk.TreeView.__init__(self)
        self.player = player

        self.model = gtk.ListStore(str, str)
        self.set_model(self.model)

        tvc = gtk.TreeViewColumn("Option", gtk.CellRendererText(), text=0)
        tvc.set_resizable(True)
        self.append_column(tvc)

        cell = gtk.CellRendererText()
        cell.set_property("editable", True)
        cell.connect("edited", self.on_edited)

        tvc = gtk.TreeViewColumn("Value", cell, text=1)
        tvc.set_resizable(True)
        self.append_column(tvc)
        
        player.client.broadcast_configval_changed(self.configval_changed_cb)

        self.populate()

    def populate(self):
        self.player.client.configval_list(self.configval_list_cb)

    def configval_list_cb(self, res):
        list = res.value().items()
        list.sort(lambda x, y: cmp(x[0], y[0]))

        for key, value in list:
            self.model.append((key, value))

    def on_edited(self, cellrenderer, path, new_text):
        iter = self.model.get_iter(path)
        key = self.model[path][0]

        self.player.client.configval_set(key, new_text)
        self.model[path][1] = new_text

    def configval_changed_cb(self, res):
        key, value = res.value().items()[0]
        for row in self.model:
            if row[0] == key:
                row[1] = value


class OptionEditor:
    """
    The base class from which other OptionEditors inherit.
    The principle is that an OptionEditor specifies an option to control and a
    widget that the user manipulates to change the option value.
    """
    def __init__(self, player, section, option, control_widget):
        self.player = player
        self.section = section
        self.option = option
        self.control = control_widget

    def set(self, value):
        """
        Store the supplied value in the config.
        """
        self.player.settings.set(self.section, self.option, value)


class BoolOptionEditor(OptionEditor):
    """
    Let a gtk.CheckButton control a config value.
    The config value is either "1" or "0".
    """
    def __init__(self, player, section, option, control_widget):
        OptionEditor.__init__(self, player, section, option, control_widget)

        old_value = bool(int(self.player.settings.get(section, option)))
        self.control.set_active(old_value)

        self.control.connect("toggled", self.on_toggled)

    def on_toggled(self, togglebutton):
        value = str(int(togglebutton.get_active()))
        self.set(value)


class ToggleListOptionEditor(OptionEditor):
    """
    Uses a gtk.TreeView as control widget.
    The TreeView has two columns:
        1.) A toggle button.
        2.) A text field.
    The resulting config string takes the form:
        field1,field2,...
    Only the field names where the toggle button is active are used.
    """
    def __init__(self, player, section, option, view, items):
        """
        @param player: The Player instance.
        @param section: The config section to control.
        @param option: The config option to control.
        @param view: A gtk.TreeView that controls the config value.
        @param items: A list of the possible items.
        """
        OptionEditor.__init__(self, player, section, option, view)

        model = gtk.ListStore(bool, str)
        view.set_model(model)
        view.set_headers_visible(False)

        # Show old settings
        visible = self.player.settings.get(section, option).split(",")
        if visible[0] != "":
            for item in visible:
                model.append((True, item))
        for item in items:
            if item not in visible:
                model.append((False, item))

        # Add the columns
        tvc_visible = gtk.TreeViewColumn("Visible", gtk.CellRendererToggle(), active=0)
        tvc_widget = gtk.TreeViewColumn("Widget", gtk.CellRendererText(), text=1)

        view.append_column(tvc_visible)
        view.append_column(tvc_widget)

        # Event handlers
        model.connect("row-deleted", self.on_row_deleted)
        view.get_columns()[0].get_cell_renderers()[0].connect("toggled", self.on_toggled)

    def on_row_deleted(self, widget, path):
        model = self.control.get_model()
        self.commit(model)

    def on_toggled(self, cellrenderer, path):
        model = self.control.get_model()
        iter = model.get_iter(path)
        model.set_value(iter, 0, not model.get_value(iter, 0))
        self.commit(model)

    def commit(self, model):
        """
        Construct config string and send it for storage.
        """
        value = ",".join([row[1] for row in model if row[0] == True])
        self.set(value)


class ColumnsOptionEditor(OptionEditor):
    """
    Let a gtk.TreeView control which columns to display.
    The config value takes the form "col1:width1,col2:width2,..."
    """
    def __init__(self, player, section, option, control_widget, columns):
        """
        @param player: The Player instance.
        @param section: The config section to control.
        @param option: The config option to control.
        @param control_widget: The TreeView that controls the config value.
        @param columns: A list of the possible columns.
        """
        OptionEditor.__init__(self, player, section, option, control_widget)
        view = control_widget
        model = gtk.ListStore(bool, int, str) # (visible, width, column)
        view.set_model(model)

        # Populate model
        visible = self.player.settings.get(section, option).split(",")
        visible_names = []
        for item in [x for x in visible if x != ""]:
            name, width = item.split(":")
            visible_names.append(name)
            model.append((True, int(width), name))
        for item in columns:
            if item not in visible_names:
                model.append((False, 100, item))

        # Add the columns
        tvc_visible = gtk.TreeViewColumn("Visible", gtk.CellRendererToggle(), active=0)
        tvc_width = gtk.TreeViewColumn("Width", gtk.CellRendererText(), text=1)
        tvc_widget = gtk.TreeViewColumn("Widget", gtk.CellRendererText(), text=2)
        tvc_width.get_cell_renderers()[0].set_property("editable", True)
        view.append_column(tvc_visible)
        view.append_column(tvc_width)
        view.append_column(tvc_widget)

        # Event handlers
        model.connect("row-deleted", self.on_row_deleted)
        view.get_columns()[0].get_cell_renderers()[0].connect("toggled", self.on_toggled)
        view.get_columns()[1].get_cell_renderers()[0].connect("edited", self.on_edited)

    def on_row_deleted(self, widget, path):
        model = self.control.get_model()
        self.commit(model)

    def on_toggled(self, cellrenderer, path):
        model = self.control.get_model()
        iter = model.get_iter(path)
        model.set_value(iter, 0, not model.get_value(iter, 0))
        self.commit(model)

    def on_edited(self, cellrenderer, path, new_text):
        try:
            new_width = int(new_text)
            new_width = max(new_width, 1)
        except ValueError:
            return
        model = self.control.get_model()
        iter = model.get_iter(path)
        model.set_value(iter, 1, new_width)
        self.commit(model)

    def commit(self, model):
        """
        Construct config string and send it for storage.
        """
        value = ",".join(["%s:%d" % (row[2], row[1]) for row in model if row[0] == True ])
        self.set(value)


class TextViewOptionEditor(OptionEditor):
    """
    Let the text in a gtk.TextView control an option in a ConfigParser.
    The text in the TextView is directly mapped to the config value.
    """
    def __init__(self, player, section, option, text_view):
        OptionEditor.__init__(self, player, section, option, text_view)

        # Show old settings
        old_value = self.player.settings.get(section, option)
        text_view.get_buffer().set_text(old_value)

        # Event handlers
        text_view.get_buffer().connect("changed", self.on_changed)

    def on_changed(self, widget):
        self.commit()

    def commit(self):
        buffer = self.control.get_buffer()
        start = buffer.get_start_iter()
        end = buffer.get_end_iter()
        text = self.control.get_buffer().get_text(start, end)
        #text = xml.sax.saxutils.escape(text)
        self.set(text)


class ColorButtonOptionEditor(OptionEditor):
    """
    Let a gtk.ColorButton control a config option.
    The stored config value takes the form "#rrggbb".
    """
    def __init__(self, player, section, option, color_button):
        OptionEditor.__init__(self, player, section, option, color_button)
        old_value = self.player.settings.get(section, option)
        gdk_color = gtk.gdk.color_parse(old_value)
        color_button.set_color(gdk_color)
        color_button.connect("color-set", self.on_color_set)


    def on_color_set(self, color_button):
        c = color_button.get_color()
        c_string = "#%04x%04x%04x" % (c.red, c.green, c.blue) 
        self.set(c_string)


class EntryOptionEditor(OptionEditor):
    """
    Let a gtk.Entry control a config option.
    The text in the gtk.Entry is directly mapped to the config value.
    """
    def __init__(self, player, section, option, entry):
        OptionEditor.__init__(self, player, section, option, entry)
        old_value = self.player.settings.get(section, option)
        entry.set_text(old_value)
        entry.connect("changed", self.on_changed)
        
    def on_changed(self, entry):
        self.set(entry.get_text())
