"""
Copyright (C) 2005 Magnus Bjernstad <bjernstad@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""

import gtk, gtk.glade, format, functions, os

ENTITY_TYPE_FILE = 0
ENTITY_TYPE_DIR = 1
ENTITY_TYPE_PARENT = 2

class SongView(gtk.ScrolledWindow):
    def __init__(self, player, section):
        gtk.ScrolledWindow.__init__(self)
        self.player = player
        self.section = section

        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)

        self.is_clickable = False
        self.cell = gtk.CellRendererText()
        self.view = gtk.TreeView()
        self.view.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
        self.add(self.view)

        player.settings.add_cb(section, "columns", self.__columns_cb)
        player.settings.add_cb(section, "custom_format", self.__custom_format_cb)
        player.settings.add_cb(section, "rules_hint", self.__rules_hint_cb)

        player.settings.emit(section, "custom_format")
        player.settings.emit(section, "rules_hint")
        player.settings.emit(section, "columns")

    def set_clickable(self, value):
        self.is_clickable = value
        for col in self.view.get_columns():
            col.set_clickable(value)

    def __columns_cb(self, section, option, value):
        self.set_columns(value) 

    def __custom_format_cb(self, section, option, value):
        self.__set_custom_format(value)
        self.view.queue_draw()

    def __rules_hint_cb(self, section, option, value):
        new_value = bool(int(value))
        self.view.set_rules_hint(new_value)

    def __set_custom_format(self, value):
        self.format = format.Format(value)

    def set_columns(self, columns):
        # Remove all old columns
        for col in self.view.get_columns():
            self.view.remove_column(col)

        # Append the new ones
        for pair in columns.split(","):
            try:
                name, width = pair.split(":")
            except ValueError:
                continue
            width = int(width)

            tvc = gtk.TreeViewColumn(name, self.cell)

            if name == "Custom":
                tvc.set_cell_data_func(self.cell, self.__custom_column_cell_data_func)
            elif name == "Duration":
                tvc.set_cell_data_func(self.cell, self.__time_column_cell_data_func)
            elif name == "Icon":
                cell = gtk.CellRendererPixbuf()
                tvc = gtk.TreeViewColumn(name, cell)
                tvc.set_cell_data_func(cell, self.__pixbuf_column_cell_data_func)
            else:
                tvc.set_cell_data_func(self.cell, self.__cell_data_func)

            tvc.set_clickable(self.is_clickable)
            tvc.set_fixed_width(width)
            tvc.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
            tvc.set_resizable(True)
            self.view.append_column(tvc)

        # Make last column autosized
        try:
            self.view.get_columns()[-1].set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
        except IndexError:
            pass

        self.view.set_headers_visible(len(self.view.get_columns()) > 1)

    def __cell_data_func(self, column, cell, model, iter):
        field = column.get_title().lower()
        try:
            value = model.get_value(iter, 0)[("server", field)]
        except KeyError:
            cell.set_property("text", "")
            return
        if field == "bitrate":
            cell.set_property("text", "%d kbps" % ( int(value) / 1000 ))
        else:
            cell.set_property("text", value)

    def __custom_column_cell_data_func(self, column, cell, model, iter):
        info_dict = model.get_value(iter, 0)
        text = self.format.format(info_dict)
        cell.set_property("markup", text)

    def __time_column_cell_data_func(self, column, cell, model, iter):
        try:
            ms = int(model.get_value(iter, 0)[("server", "duration")])
            string = "%2d:%02d" % (ms/60000, (ms/1000)%60)
            cell.set_property("text", string)
        except KeyError:
            cell.set_property("text", "")

    def __pixbuf_column_cell_data_func(self, column, cell, model, iter):
        entity_type = model.get_value(iter, 1)
        if entity_type == ENTITY_TYPE_FILE:
            cell.set_property("stock_id", gtk.STOCK_NEW)
        elif entity_type == ENTITY_TYPE_DIR:
            cell.set_property("stock_id", gtk.STOCK_OPEN)
        elif entity_type == ENTITY_TYPE_PARENT:
            cell.set_property("stock_id", "")

    def fill_begin(self):
        self.view.set_sensitive(False)

    def fill_end(self):
        functions.queue_function(self.player, self.view.set_sensitive, True)

    def destructor(self):
        self.player.settings.remove_cb(self.section, "columns", self.__columns_cb)
        self.player.settings.remove_cb(self.section, "custom_format", self.__custom_format_cb)
        self.player.settings.remove_cb(self.section, "rules_hint", self.__rules_hint_cb)


class TagInfo:
    def __init__(self, info):
        glade_file = functions.glade_file("tag_info")
        gxml = gtk.glade.XML(glade_file, "tag_info")
        gxml.signal_autoconnect(self)

        tags = ["url", "title", "artist", "album", "comment", "date",
        "tracknr", "genre", "mime", "bitrate", "samplerate"]

        w = gxml.get_widget
        for tag in tags:
            self.show_tag(w, tag, info)

    def show_tag(self, w, tag, info):
        try:
            data = info.get(("server", tag))
            if data == None:
                return
            w(tag).set_text(str(data))
        except TypeError:
            pass
    
    def on_close(self, widget):
        widget.get_toplevel().destroy()

    def on_key_pressed(self, widget, event):
        if event.keyval == gtk.gdk.keyval_from_name("Escape"):
            widget.get_toplevel().destroy()
